#include<iostream>
#include<sstream>
#include<string>
using namespace std;

//GapFilter
//Checks whether if a PSL alignment provides large gaps to be blatted again within the contig, intron, gene, etc.


int gapThreshold = 20; //any gap >=threshold 

bool FilterCheck(stringstream& str)
{
	int matches, mismatches;
	str >> matches >> mismatches;

	int junk;
	str >> junk >> junk >> junk >> junk >> junk >> junk;
	
	string junks;
	str >> junks >> junks;
	
	int querySize;
	str >> querySize;
	
	if(querySize - matches - mismatches < gapThreshold/* || matches + mismatches < gapThreshold*/) ///Commented out filter only belongs to duplication event (moved to blatter side in WI version)
	{
		//cout << "Returned Negative in initial filtering" << endl;
		return 0;
	}

	str >> junk >> junk >> junks >> junk >> junk >> junk;
	
	//filtering by blocks
	int blockCount; 
	str >> blockCount;
	
	int* blockSizes;
	int* qIndices;
	
	blockSizes = (int*) malloc ((blockCount+3)*sizeof(int));
	qIndices = (int*) malloc ((blockCount+3)*sizeof(int));
	
	string blockSizesStr, qIndicesStr;
	str >> blockSizesStr;
	str >> qIndicesStr;
	
	int afterCommaIndex = 0;
	for(int i=1; i<=blockCount; i++)
	{
		int nextCommaPos = blockSizesStr.find(",", afterCommaIndex);
		
		string tempStr = blockSizesStr.substr(afterCommaIndex, nextCommaPos - afterCommaIndex);
		blockSizes[i] = atoi(tempStr.c_str());
		
		afterCommaIndex = nextCommaPos + 1;
	}
	
	afterCommaIndex = 0;
	for(int i=1; i<=blockCount; i++)
	{
		int nextCommaPos = qIndicesStr.find(",", afterCommaIndex);
		
		string tempStr = qIndicesStr.substr(afterCommaIndex, nextCommaPos - afterCommaIndex);
		qIndices[i] = atoi(tempStr.c_str());
		
		afterCommaIndex = nextCommaPos + 1;
	}

	blockSizes[0] = 0; qIndices[0]=0; 					//For detecting the gap before the first block
	blockSizes[blockCount+1] = 0; qIndices[blockCount+1] = querySize; 	//For detecting the gap after the last block	

	for(int i=1; i<=blockCount+1; i++)	//index(1) looks for any gap before first block - index(blockCount+1) looks for any gap after the last block
	{
		int gapSize = qIndices[i] - (qIndices[i-1] + blockSizes[i-1]);
		if(/*gapSize <= matches + mismatches &&*/ gapSize >= gapThreshold) ///Commented out filter only belongs to duplication event  (moved to blatter side in WI version)
		{
			//cout << "Returned Positive" << endl;
			free(blockSizes);
			free(qIndices);
			return 1;
		}	
	}	
	//cout << "Returned Negative" << endl;	
	free(blockSizes);
	free(qIndices);
	return 0;
}

int main(int argc, char *argv[])
{
	if(argc < 2)
	{
		cout << "ERROR: There is a required argument -> 1[PSL_LINE]" << endl;
		cout << "       Optional argument: 2[int(gapThreshold)]" << endl;
		exit(250);
	}

        if (string("-v") == string(argv[1]))
        {
          cout << "GapFilterWI Version: 1.0" << endl;
          exit(0);
        }

	stringstream str(argv[1]);
	
	if(argc > 2)
		gapThreshold = atoi(argv[2]);

	if(FilterCheck(str))
		exit(1);
	else
		exit(0);
} 

