from align_parsers import exonerate,blat
import sys, os, re
from optparse import OptionParser
import assembly

#generic alignment parsing functions not specific to particular aligner

def parse(file, aligner, filters=None, splice_motif_file=None, noline=None):
    print file, aligner

    #do filtering at the parsing level will reduce creating unwanted objects
    #do sorting here because all alignments are the same regardless of aligners
    aligns = {
        'exonerate': exonerate.parse,
        'blat': blat.parse,
        }[aligner](file, filters, splice_motif_file=splice_motif_file, noline=noline)

    print "total:", len(aligns)
    
    return aligns

def get_targets(file_name):
    "get custom targets"

    targets = []
    f = open(file_name, 'r')
    for line in f.readlines():
        if re.search('\d', line):
             (label, target) = re.split('\t+', line.rstrip('\n').lstrip(' '))
             target = re.sub('(chr|,)', '', target)
             targets.append(target)
    f.close()

    return targets

def get_snvs(aligns, aligner, blat_blast_output=None):
    all_snvs = []
    
    if aligner.lower() == "exonerate":
        for align in aligns:
            snvs = exonerate.get_snvs(align)
            
            if snvs:
                #setting position from end
                for snv in snvs:
                    snv.var_len = align.query_len
                    snv.from_end = min(int(snv.var_start) - int(align.qstart), int(align.qend) - int(snv.var_end))
                    
                all_snvs.extend(snvs)

    elif aligner.lower() == "blat":
        if blat_blast_output:
            hsps = blat.get_hsps(blat_blast_output, aligns)
            sys.stderr.write("number of hsps:%d\n" % (len(hsps)))

            #mutable
            hsps_used = {}

            count_block = 0
            count_hsp = 0
            for align in aligns:
                snvs, hsps_found = blat.get_snvs(align, hsps, hsps_used)
                count_hsp += hsps_found
                count_block += len(align.blocks)

                if snvs:
                    for snv in snvs:
                        snv.var_len = align.query_len
                        snv.from_end = min(int(snv.var_start) - int(align.qstart), int(align.qend) - int(snv.var_end))
                    
                    all_snvs.extend(snvs)
                
            sys.stderr.write("blat snvs blocks:%d hsps:%d\n" % (count_block, count_hsp))

        
    return all_snvs

def output_aligns(outfile, aligns, track_name=None, contigs=None):
    ext = os.path.splitext(outfile)[1]
    out = open(outfile, 'w')

    track_desc = track_name
    #track_name = track_desc

    #alignments - output track
    if aligns:
        if track_name:
            out.write("track name=\"%s\" description=\"%s\" visibility=%d itemRgb=\"On\"\n" % (track_name, track_desc, 3))

        #map contig to contig name
        contig_dict = None
        if contigs:
            contig_dict = dict((contig.num, contig) for contig in contigs)
            
        for align in aligns:
            #this is for adding coverage info
            contig = align.query
            if ' ' in contig:
                contig = align.query.split(" ")[0]
            
            if contig_dict and contig_dict.has_key(contig):
                align.contig = contig_dict[contig]
                
            if ext == ".gff":
                out.write(align.gff("exon"))

            elif ext == ".psl":
                out.write(align.psl())

            elif ext == ".exon":
                out.write(align.exon())
                
    out.close()

def output_snvs(outfile, snvs, track_name=None):
    ext = os.path.splitext(outfile)[1]
    
    track_desc = track_name

    out = open(outfile, 'w')

    if not ext == ".snv":
        if track_name:
            out.write("track name=\"%s SNVs\" description=\"%s SNVs\" visibility=%d itemRgb=\"On\"\n" % (track_name, track_desc, 3))
    else:
        out.write("\t".join(['type', 'chr', 'chr_start', 'chr_end', 'strand', 'ctg', 'ctg_len', 'ctg_start', 'ctg_end', 'len', 'change', 'from_end']) + "\n")

    if snvs:
        snvs_snvs = [snv for snv in snvs if snv.snv_type and snv.snv_type.lower() == 'snv']
        
        for snv in snvs_snvs:
            if not ext == ".snv":
                out.write(snv.gff("exon"))
            elif ext == ".snv":
                out.write(snv.tab())
            
        snvs_indels = [snv for snv in snvs if snv.snv_type and not snv.snv_type.lower() == 'snv']
            
        for snv in snvs_indels:
            if not ext == ".snv":
                out.write(snv.gff("exon"))
            elif ext == ".snv":
                out.write(snv.tab())
                
        out.close()

def get_contigs(fasta, kmer=None):
    sys.stderr.write("extracting contigs...")

    if kmer:
        ass = assembly.Assembly(k=kmer)
    else:
        ass = assembly.Assembly(k=1)
        
    ass.fasta = fasta
    contigs = ass.get_contigs()

    for contig in contigs:
        if ":" in contig.num:
            k, ctg = contig.num.split(':')
            contig.k = k.lstrip('k')
        elif kmer:
            contig.k = kmer
        #print "test",contig.num, contig.coverage, contig.k, contig.length
    sys.stderr.write("done\n")
    return contigs
    

if __name__ == '__main__':
    usage = "Usage: %prog file aligner [options]"
    parser = OptionParser(usage=usage)
    parser.add_option("-n", "--bestn", dest="bestn", help="best hit filter")
    parser.add_option("-i", "--identity", dest="identity", help="identity filter")
    parser.add_option("-m", "--match", dest="match_percent", help="match length percentage")
    parser.add_option("-u", "--unique", dest="unique", help="unique", action="store_true", default=False)
    parser.add_option("-q", "--query_len", dest="query_len", help="query length filter")
    parser.add_option("-t", "--target", dest="target", help="target e.g. chr1:35422000-35425800, chr13, or file containing targets")
    parser.add_option("-s", "--split", dest="split", help="split alignments", action="store_true", default=False)
    parser.add_option("-v", "--snvs", dest="snvs", help="get snvs", action="store_true", default=False)
    parser.add_option("-o", "--out", dest="out", help="output file, with proper file extension e.g. x.gff, x.psl, x.snv, or x.gff,x.snv")
    parser.add_option("-k", "--track", dest="track_name", help="track name e.g. \"HS1202 k45\"")
    parser.add_option("-w", "--snv_out", dest="snv_out", help="output .snv file")
    parser.add_option("-b", "--blast", dest="blast", help="blat blast output for snv detection")
    parser.add_option("-f", "--fasta", dest="fasta", help="fasta file, for extracting contig info")
    parser.add_option("-K", "--k", dest="k", help="kmer value, if not present in contig name")
    parser.add_option("-S", "--s", dest="splice", help="splice motif file", default="/home/rchiu/work/wtss/splicing/splice_motives.txt")

    (options, args) = parser.parse_args()
    
    filters = {}
    if len(args) == 2:
        if options.bestn:
            filters['bestn'] = int(options.bestn)
        if options.identity:
            filters['identity'] = float(options.identity)
        if options.match_percent:
            filters['match'] = float(options.match_percent)
        if options.unique:
            filters['unique'] = True
        if options.query_len:
            filters['qlen'] = int(options.query_len)
        if options.target:
            if os.path.exists(options.target):
                filters['target'] = get_targets(options.target)
            else:
                filters['target'] = [options.target]

        #get contig/coverage
        contigs = None
        if options.fasta:
            contigs = get_contigs(options.fasta, options.k)

        #aligns
        #this is specifically for outputting exonerate original alignments
        if options.out and os.path.splitext(options.out)[1] == ".exon":
            aligns = parse(args[0], args[1], filters, splice_motif_file=options.splice, noline=False)
        else:
            aligns = parse(args[0], args[1], filters, splice_motif_file=options.splice)
            
        if options.out:
            output_aligns(options.out, aligns, options.track_name, contigs=contigs)

        #snvs
        snvs = []
        if options.snvs:
            if args[1].lower() == "blat" and options.blast:
                snvs = get_snvs(aligns, args[1], options.blast)
            else:
                snvs = get_snvs(aligns, args[1])

            if options.out:
                (filename, ext) = os.path.splitext(options.out)
                outfile = filename + ".snv" + ext

                output_snvs(outfile, snvs, options.track_name)

            if options.snv_out:
                output_snvs(options.snv_out, snvs)

    else:
        parser.error("incorrect number of arguments")
