from assembly import Contig
from snv import SNV

class Alignment:

    line = None
    method = None
    query = target = None
    query_len = target_len = None
    #strand = Bolean
    target_strand = None
    query_strand = None
    qstart = qend = tstart = tend = None
    blocks = None
    splice_sites = None
    query_blocks = None
    match_len = matches = mismatches = None
    identity = None
    score = None
    rank = None
    model = None
    pairwise = None
    psl_str = None
    #transcript orientation inferred from Exonerate alignment
    orient = None
    contig = None

    def __init__(self, method):
        self.method = method
        self.blocks = self.query_blocks = self.splice_sites = []

    def details(self):
        if self.blocks:
            return "%s %s-%s %s %s-%s %s" % (self.query, self.qstart, self.qend, self.target, self.tstart, self.tend, len(self.blocks))
        else:
            return "%s %s-%s %s %s-%s" % (self.query, self.qstart, self.qend, self.target, self.tstart, self.tend)
        
    def gff(self, type):
        if not self.blocks:
            print "no blocks", self.query, self.target
            return ""
        
        #chromosome
        if 'chr' in self.target:
            chrom = self.target
        else:
            chrom = 'chr' + self.target

        #contig
        contig = self.query.split(" ")[0]
        if self.contig and self.contig.num:
            contig = str(self.contig.num)

        #strand
        if self.target_strand:
            strand = "+"
        else:
            strand = "-"

        #gene orientation
        orient = "?"
        if self.orient:
            if self.orient == 'forward':
                orient = "+"
            elif self.orient == 'backward':
                orient = "-"

        #coverage
        cov = None
        if self.contig:
            cov = str(self.contig.normalized_coverage())

        if cov:
            group = ":".join([self.target, str(self.tstart), "%s%s" % (strand, orient), contig, str(cov)])
        else:
            group = ":".join([self.target, str(self.tstart), "%s%s" % (strand, orient), contig])

        #do this after recording strand info in group
        if orient != "?":
            strand = orient

        out = ""
        
        for i in range(len(self.blocks)):
            qstart, qend = self.query_blocks[i]
            tstart, tend = self.blocks[i]
            
            splice_site = ""
            if len(self.blocks) > 1 and i < len(self.blocks)-1 and len(self.blocks):
                splice_site = self.splice_sites[i]

            #hide query block and splice sites in "program" field
            annot = ":".join([str(qstart), str(qend), splice_site])

            out += "\t".join([chrom, annot, type, tstart, tend, ".", strand, ".", group]) + "\n"

        return out

            

    def psl(self):
        "output ucsc psl format"

        if self.psl:
            psl_str = self.psl_str

            cols = self.psl_str.split("\t")
            #make sure target has chr in it for UCSC
            if not 'chr' in cols[13]:
                cols[13] = 'chr' + cols[13]
                psl_str = "\t".join(cols)
            
            if self.contig:
                cov = self.contig.normalized_coverage()
                if cov:
                    contig = ":".join([str(self.contig.num), str(cov)])
                    cols[9] = contig

                    psl_str = "\t".join(cols)
                
            return psl_str + "\n"

    def exon(self):
        "output original exonerate record"

        if self.record:
            return self.record + "\n"


from string import maketrans

class SNVfinder:

    def __init__(self, align):
        self.align = align
        self.complement = maketrans("agtcAGTC", "tcagTCAG")

    def find_snv(self, seqs, aln, starts, query_strand, target_strand, method):
        "find indels in contiguous pairwise alignment"

        if len(seqs) != 2 or (len(seqs[0]) != len(seqs[1]) and lens(seqs[0]) != len(alns)):
            return None

        SNVs = []

        i = q = t = 0
        while i < len(seqs[0]):
            dels = []
            dels_qbase = []
            dels_tbase = []
            ins = []
            ins_qbase = []
            ins_tbase = []
            j = i
            snvs = []
            snvs_qbase = []
            snvs_tbase = []
            event = None
            while j < len(seqs[0]):
                #base match
                if seqs[0][j].upper() == seqs[1][j].upper():
                    t += 1
                    q += 1
                    break
                #del
                elif seqs[0][j] == '-':
                    event = 'del'
                    dels.append(j)
                    dels_qbase.append(q)
                    dels_tbase.append(t)
                    t += 1
                #ins
                elif seqs[1][j] == '-':
                    event = 'ins'
                    ins.append(j)
                    ins_qbase.append(q)
                    ins_tbase.append(t)
                    q += 1
                #snp
                else:
                    event = 'snv'
                    snvs.append(j)
                    snvs_qbase.append(q)
                    snvs_tbase.append(t)
                    t += 1
                    q += 1
                j += 1

            strand = target_strand
            if method == "blat":
                strand = query_strand

            if snvs:
                #print "snvs:", i, j, snvs, snvs_qbase, snvs_tbase

                query = seqs[0][snvs[0]:snvs[-1]+1]
                target = seqs[1][snvs[0]:snvs[-1]+1]

                if not target_strand:
                    query = query[::-1].translate(self.complement)
                    target = target[::-1].translate(self.complement)

                if query_strand:
                    (qstart, qend) = (int(starts[0])+int(snvs_qbase[0]), int(starts[0])+int(snvs_qbase[-1]))
                else:
                    (qstart, qend) = (int(starts[0])-int(snvs_qbase[-1]), int(starts[0])-int(snvs_qbase[0]))

                if target_strand:
                    (tstart, tend) = (int(starts[1])+int(snvs_tbase[0]), int(starts[1])+int(snvs_tbase[-1]))
                else:
                    (tstart, tend) = (int(starts[1])-int(snvs_tbase[-1]), int(starts[1])-int(snvs_tbase[0]))
                    
                #print "snvs:", seqs[0][i], seqs[1][i], i, snvs, query, target, query_strand, target_strand, method
                #print "SNP %s:%d-%d %s:%d-%d %d %s->%s" % (self.align.target, tstart, tend, self.align.query, qstart, qend, len(snvs), target, query)
                
                SNVs.append(SNV(method, "snv", self.align.target, tstart, tend, target, strand, self.align.query, qstart, qend, query))
                #print SNVs[-1].details()

            if ins:
                #print "ins:", i, j, ins, ins_qbase, ins_tbase

                query = seqs[0][ins[0]:ins[-1]+1]
                target = seqs[1][ins[0]:ins[0]+len(query)]

                if not target_strand:
                    query = query[::-1].translate(self.complement)

                if query_strand:
                    (qstart, qend) = (int(starts[0])+int(ins_qbase[0]), int(starts[0])+int(ins_qbase[-1]))
                else:
                    (qstart, qend) = (int(starts[0])-int(ins_qbase[0]), int(starts[0])-int(ins_qbase[0]))

                if target_strand:
                    (tstart, tend) = (int(starts[1])+int(ins_tbase[0])-1, int(starts[1])+int(ins_tbase[0])-1)
                else:
                    (tstart, tend) = (int(starts[1])-int(ins_tbase[0]), int(starts[1])-int(ins_tbase[0]))
                    
                #print "ins:", seqs[0][i], i, ins, query, target, query_strand, target_strand, method
                #print "INS %s:%d-%d %s:%d-%d %d %s->%s" % (self.align.target, tstart, tend, self.align.query, qstart, qend, len(ins), target, query)
                SNVs.append(SNV(method, "ins", self.align.target, tstart, tend, target, strand, self.align.query, qstart, qend, query))
                #print SNVs[-1].details()
                
            if dels:
                #print "dels:", i, j,  dels, dels_qbase, dels_tbase

                target = seqs[1][dels[0]:dels[-1]+1]
                query = seqs[0][dels[0]:dels[0]+len(target)]
                
                if not target_strand:
                    target = target[::-1].translate(self.complement)

                if query_strand:
                    (qstart, qend) = (int(starts[0])+int(dels_qbase[0]-1), int(starts[0])+int(dels_qbase[0]-1))
                else:
                    (qstart, qend) = (int(starts[0])-int(dels_qbase[0]), int(starts[0])-int(dels_qbase[0]))

                if target_strand:
                    (tstart, tend) = (int(starts[1])+int(dels_tbase[0]), int(starts[1])+int(dels_tbase[-1]))
                else:
                    (tstart, tend) = (int(starts[1])-int(dels_tbase[-1]), int(starts[1])-int(dels_tbase[0]))
                    
                #print "dels:", seqs[0][i], i, dels, query, target, query_strand, target_strand, method
                #print "DEL %s:%d-%d %s:%d-%d %d %s->%s" % (self.align.target, tstart, tend, self.align.query, qstart, qend, len(dels), target, query)
                SNVs.append(SNV(method, "del", self.align.target, tstart, tend, target, strand, self.align.query, qstart, qend, query))
                #print SNVs[-1].details()
                
            i += len(ins) + len(dels) + len(snvs) + 1

        return SNVs



    
    
    
