__author__ = "Readman Chiu"
__version__ = "0.1"

import os, sys, tempfile, ConfigParser, random, glob
from shutil import copy
from optparse import OptionParser
import assembly

class ClusterAlign:
    "run alignments on the cluster"
    
    name = None
    #can be full path
    aligner = None
    inputs = []
    input_file = None
    target = None
    out_dir = None
    jobs_file = None
    #make top_dir an optional parameter?
    top_dir = None
    config_file = None
    config = None

    def __init__(self, name, aligner, config_file, input_file=None, target=None, out_dir=None, assembly=None, top_dir=None):
        self.name = name
        self.aligner = aligner
        self.input_file = input_file
        self.target = target
        self.out_dir = out_dir

        self.config_file = config_file
        self.config = ConfigParser.ConfigParser()
        self.config.read(self.config_file)

        #set up directory
        if not out_dir:
            if top_dir:
                self.top_dir = top_dir + "/cluster"
            elif assembly:
                self.top_dir = assembly.loc + "/cluster"

            if self.top_dir:
                #create topdir if not exists
                if not os.path.exists(self.top_dir):
                    os.mkdir(self.top_dir)

                if not os.path.exists(self.top_dir + "/" + name):
                    os.mkdir(self.top_dir + "/" + name)
                    os.mkdir(self.top_dir + "/" + name + "/jobs")
                    os.mkdir(self.top_dir + "/" + name + "/input")
                    os.mkdir(self.top_dir + "/" + name + "/output")

    def prepare_input(self, queries_per_job, out_dir=None, contigs=[], input_dir=None):
        "prepare input fasta files"

        #if out_dir is not specifically given and is following file structure...
        if not out_dir and self.top_dir:
            out_dir = self.top_dir + "/" + self.name + "/input"
        
        #link to input dir if provided
        if input_dir:
            if os.path.islink(out_dir):
                os.remove(out_dir)
            elif os.path.isdir(out_dir):
                os.rmdir(out_dir)
            os.symlink(input_dir, out_dir)

            for input_file in glob.glob(os.path.join(input_dir,'*.fa')):
                self.inputs.append(input_file)

        else:
            if not os.path.isdir(out_dir):
                os.mkdir(out_dir)

            lines = []
            seq_count = 0
            file_count = 1
            files = []
            order_indices = range(len(contigs))
            random.shuffle(order_indices)
            count = 0
            for i in order_indices:
                lines.append(contigs[i].fasta())
                seq_count = seq_count + 1

                if seq_count == queries_per_job or count == len(contigs)-1:
                    out_file = out_dir + "/seq." + str(file_count) + ".fa"
                    out = open(out_file, 'w')
                    out.writelines(lines)
                    out.close()
                    self.inputs.append(out_file)
                
                    file_count = file_count + 1
                
                    #reset counter
                    seq_count = 0
                    lines = []

                count = count + 1

    def create_jobs(self, jobs_file=None, target=None, out_dir=None):
        "create batch file of cluster jobs"

        if not jobs_file and self.top_dir:
            self.jobs_file = self.top_dir + "/" + self.name + "/jobs/jobs"
        else:
            self.jobs_file = jobs_file
            
        if not out_dir and self.top_dir:
            out_dir = self.top_dir + "/" + self.name + "/output"
            if not os.path.exists(out_dir):
                os.mkdir(out_dir)

        #remove old file if it's there
        if os.path.exists(self.jobs_file):
            os.remove(self.jobs_file)

        #figure out target file
        target_files = {}
        if os.path.exists(target) and "/" in target:
            target_files[os.path.basename(target)] = target
        else:
            target_files = self.get_target_file(target)
                
        if len(target_files.keys()) < 1 or not os.path.exists(out_dir):
            return False

        #create one job for each fasta input
        lines = []
        for target_name,target_file in target_files.iteritems():
            #prepare target copy(copy of target file in var/tmp
            target_copy = os.path.basename(target_file)
            
            for fasta in self.inputs:
                line = "time python " + sys.modules[__name__].__file__ + " -a " + self.aligner + " -n " + self.name + " -f " + fasta + " -t " + target_file + " -o " + out_dir + " -c " + self.config_file + "\n"
                lines.append(line)

        #write batch jobs file
        print self.jobs_file
        out = open(self.jobs_file, 'w')
        out.writelines(lines)
        out.close()

        return True

    def get_target_file(self, target):
        "get target/db files from config file"
        
        target_files = {}
        
        (organism, genome) = target.split('_')
        if genome == 'genome' or genome == 'transcriptome':
            section = 'align-' + genome
            if genome == 'genome':
                option = [o for o in self.config.options('align-genome') if organism in o and len(o) > len(organism)]
            else:
                option = [o for o in self.config.options('align-transcriptome') if organism in o and len(o) > len(organism)]
            
        else:
            if target in self.config.options('align-transcriptome'):
                (section, option) = ('align-transcriptome', target)
            elif target in self.config.options('align-genome'):
                (section, option) = ('align-genome', target)

        if type(option).__name__ == 'list':
            for o in option:
                #use local "copy" of genome files instead
                if genome == 'genome' and 'align-genome-copy' in self.config.sections() and o in self.config.options('align-genome-copy'):
                    section = 'align-genome-copy'
                target_files[o] = self.config.get(section, o)
        else:
            #use local "copy" of genome files instead
            if 'align-genome-copy' in self.config.sections() and option in self.config.options('align-genome-copy'):
                section = 'align-genome-copy'
            target_files[option] = self.config.get(section, option)

        return target_files 
            
    def submit(self, jobs_file=None, mem=None):
        "submit batch alignment jobs through mqsub"

        if not jobs_file:
            jobs_file = self.jobs_file

        jobs_dir = os.path.dirname(self.jobs_file)

        shell_script = self.config.get("align-jobs", "submit_script")
        
        if os.path.exists(jobs_file) and os.path.exists(shell_script) and os.path.exists(jobs_dir):
            command = shell_script + " " + jobs_dir + " " + jobs_file + " " + self.name + " " + mem
            #print command
            os.system(command)               
    
    def run(self):
        "run an alignment job on cluster node"

        if not self.input_file or not self.target or not self.out_dir or not self.aligner:
            return False

        #copy query to local directories using tempfile
        if (os.path.exists(self.input_file)):
            query = tempfile.NamedTemporaryFile()
            copy(self.input_file, query.name)

        #copy target to local, if target is a file (if target is not a file, should get target path from configs
        if (not os.path.exists(self.target)):
            #check if file already copied to /var/tmp
            target = tempfile.NamedTemporaryFile()
            copy(self.target, target.name)
            self.target = target.name

        #make sure output directory is there
        if not os.path.isdir(self.out_dir):
            os.mkdir(self.out_dir)
            
        #prepare output file
        suffix = self.config.get("align-" + self.aligner, "suffix")
        out_file = os.path.splitext(os.path.basename(self.input_file))[0]
        out_file = self.out_dir + "/" + out_file + "." + suffix
        
        #prepare final command
        command = self.config.get("align-" + self.aligner, "command")
        command = command.replace('QUERY', query.name).replace('TARGET', self.target).replace('OUTPUT', out_file)

        #submit jobs
        if command and os.path.getsize(query.name) > 0 and os.path.getsize(self.target) > 0:
            os.system(command)

    def batch(self, queries_per_job, submit, k=None, input_dir=None, mem=None):
        "given a fasta files, split input and run alignment on cluster"

        contigs = None
        if not input_dir:
            ab = assembly.Assembly(k, fasta = self.input_file)
            contigs = ab.get_contigs(sequence=True)
            sys.stderr.write("contigs parsed ... %d" % (len(contigs)))

        if contigs or input_dir:
            self.prepare_input(queries_per_job=int(queries_per_job), contigs=contigs, input_dir=input_dir)
            success = self.create_jobs(target=self.target)

            if success:
                print "successfully created jobs"
                if submit:
                    self.submit(mem=mem)

if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option("-a", "--aligner", dest="aligner", help="blat|exonerate")
    parser.add_option("-n", "--name", dest="name", help="name of jobs")
    parser.add_option("-f", "--fasta", dest="fasta_file", help="fasta file of input")
    parser.add_option("-t", "--target", dest="target", help="database file")
    parser.add_option("-o", "--output", dest="out_dir", help="output directory of alignments")
    parser.add_option("-s", "--submit", dest="submit", help="submit jobs", action="store_true", default=False)
    #additional options for running batch jobs
    parser.add_option("-b", "--batch", dest="batch", help="batch submit jobs", action="store_true", default=False)
    parser.add_option("-q", "--queries_per_job", dest="queries_per_job", help="queries_per_job")
    parser.add_option("-k", "--k_value", dest="k", help="assembly k value")
    parser.add_option("-c", "--config", dest="config", help="config file")
    parser.add_option("-i", "--input", dest="input_dir", help="provides input files(dir)")
    parser.add_option("-m", "--mem", dest="mem", help="memory required, exonerate=12.2G, blat=5G(default)", default="5G")
    
    (options, args) = parser.parse_args()

    if not options.config:
        config_file = "/home/rchiu/work/wtss/wtss/wtss.cfg"
    else:
        config_file = options.config

    if options.batch:
        jobs = ClusterAlign(name=options.name, aligner=options.aligner, config_file=config_file, input_file=options.fasta_file, target=options.target, top_dir=options.out_dir)
        jobs.batch(queries_per_job=options.queries_per_job, submit=options.submit, k=options.k, input_dir=options.input_dir, mem=options.mem)
    else:
        job = ClusterAlign(name=options.name, aligner=options.aligner, config_file=config_file, input_file=options.fasta_file, target=options.target, out_dir=options.out_dir)
        job.run()
    

    
