import alignment
from optparse import OptionParser
import re
import os
from align_parsers import blat

class Track:

    track = None
    format = None
    name = None
    desc = None

    def __init__(self, track=None):
        self.track = track

    def parse_gff(self):
        record = []
        aligns = []
        
        start = False
        for line in open(self.track, 'r'):
        
            if line[:3].lower() == "chr":
                fields = line.split("\t")

                #hack for merged contigs!
                if len(fields) == 5:
                    fields = [fields[0]+fields[1],fields[2],fields[3],fields[4]]

                if not record or record[-1][-1] == fields[-1]:
                    record.append(fields)
                elif record:
                    record.sort(lambda x,y: int(x[3])-int(y[3]))
                    blocks = []
                    splice_sites = []
                    query_blocks = []
                    for r in record:
                        blocks.append([r[3], r[4]])

                        if ":" in r[1]:
                            qstart, qend, splice_site = r[1].split(":")
                            query_blocks.append([qstart, qend])
                            splice_sites.append(splice_site)
                    
                    align = self.create_align(record[0][1], record[0][-1], record[0][0], record[0][6], blocks)
                    align.query_blocks = query_blocks
                    align.splice_sites = splice_sites
                    aligns.append(align)
                    
##                    print align.target, align.query, align.target_strand, len(align.blocks), len(align.query_blocks), len(align.splice_sites), align.orient
##                    print align.blocks
##                    print align.query_blocks
##                    print align.splice_sites
##                    print "---"
                    

                    record = [fields]
        if record:
            record.sort(lambda x,y: int(x[3])-int(y[3]))
            blocks = []
            splice_sites = []
            query_blocks = []
            for r in record:
                blocks.append([r[3], r[4]])
                
                if ":" in r[1]:
                    qstart, qend, splice_site = r[1].split(":")
                    query_blocks.append([qstart, qend])
                    splice_sites.append(splice_site)
                
            align = self.create_align(record[0][1], record[0][-1], record[0][0], record[0][6], blocks)
            align.query_blocks = query_blocks
            align.splice_sites = splice_sites
            aligns.append(align)

##            print align.target, align.query, align.target_strand
##            print align.blocks
##            print align.query_blocks
##            print align.splice_sites
##            print "---"
                
        return aligns

    def parse_psl(self):
        aligns = blat.parse(self.track)

        for align in aligns:
            align.query, align.coverage, align.orient, strand = self.get_contig_name(align.query)
        
        return aligns

    def create_align(self, method, query=None, target=None, strand=None, blocks=None):
        align = alignment.Alignment(method)

        align.query, align.coverage, align.orient, strand = self.get_contig_name(query)
        align.target = target

        if strand == '+':
            align.target_strand = True
        else:
            align.target_strand = False

        align.blocks = blocks

        align.tstart = blocks[0][0]
        align.tend = blocks[-1][-1]

        return align

    def get_contig_name(self, label):
        contig = label.rstrip()
        coverage = None
        orient = None
        strand = None
        
        if re.match("^\d+", label) and not "k" in label:
            fields = re.split(':', label.rstrip())
            contig = fields[0]
            
            if len(fields) > 1:
                coverage = fields[1]
                
        elif re.match("k\d+:\d+$", label):
            pass
        else:
            fields = re.split(':', label.rstrip())
            #print "fields",fields
            if len(fields) >= 3 and fields[-3][0] == 'k':
                contig = ':'.join(fields[-3:-1])
                coverage = fields[-1]

                #get transcript orientation if it's there in gff file
                if len(fields) > 3:
                    orient = fields[-4][-1]
                    strand = fields[-4][0]
                    
            elif len(fields) >= 2 and fields[-2][0] == 'k':
                contig = ':'.join(fields[-2:])

        return contig, coverage, orient, strand
                

    def output(self, aligns, outfile):
        ext = os.path.splitext(outfile)[1]
        out = open(outfile, 'w')

        out.write("track name=\"%s\" description=\"%s\" visibility=%d itemRgb=\"On\"\n" % (self.name, self.desc, 3))

        for align in aligns:
            if ext == ".gff":
                out.write(align.gff("exon"))

    def output_multiple(self, tracks, aligns, outfile):
        if len(tracks) == len(aligns):
            ext = os.path.splitext(outfile)[1]
            out = open(outfile, 'w')

            for i in range(len(tracks)):
                out.write("track name=\"%s\" description=\"%s\" visibility=%d itemRgb=\"On\"\n" % (tracks[i].name, tracks[i].desc, 3))

                for align in aligns[i]:
                    if ext == ".gff":
                        out.write(align.gff("exon"))

if __name__ == '__main__':
    usage = "Usage: %prog file"
    parser = OptionParser(usage=usage)

    (options, args) = parser.parse_args()

    if len(args) == 1:
        track = Track(args[0])
        track.parse_gff()
    else:
        parser.error("incorrect number of arguments")

     
